#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/power.h>

#include "LUFA/Version.h"
#include "LUFA/Drivers/USB/USB.h"
#include "LUFA/Drivers/USB/Class/MIDI.h"

#include "usb_midi_descriptors.h"

#include "midi.h"
#include "ghetto_drum.h"

volatile uint16_t trigger_state = 0x0000;
volatile uint16_t last_trigger_state = 0x0000;
volatile bool trigger_input_changed = false;

int main(void) {
	
	/* Disable watchdog if enabled by bootloader/fuses */
	MCUSR &= ~(1 << WDRF);
	wdt_disable();

	/* Disable clock division */
	clock_prescale_set(clock_div_1);
	
	// Enable pull-up bus power by turning off PUD (pull-up disable).
	MCUCR &= ~_BV(PUD);

	init_triggers();
		
	/* Initialize USB Subsystem */
	USB_Init();

	int ct =0;
	/* Main loop */
	while(1){

		MIDI_EventPacket_t ReceivedMIDIEvent;

		scan_triggers();
		check_and_handle_midi();

		USB_USBTask();
	}
}

void init_triggers(){
	TRIGGER_DDR = 0b00000000;	//all 8 inputs

	//TODO: Consider using pullups, but probably not required.
	//Enable pullups on all row pins (inputs)
	//TRIGGER_PORT = _BV(KP_ROW1_PIN) | _BV(KP_ROW2_PIN) | _BV(KP_ROW3_PIN) | _BV(KP_ROW4_PIN);
}


void scan_triggers() {

	if(trigger_input_changed){		//still waiting to send last change, don't rescan
		return;
	}

	last_trigger_state = trigger_state;
	trigger_state = TRIGGER_PINS;
	
	if(trigger_state != last_trigger_state){
		trigger_input_changed = true;
	}
}

/* Handles the input and output of MIDI messages. */
void check_and_handle_midi() {

	/* Select the MIDI IN stream */
	// Believed to be IN as seen by the host.
	Endpoint_SelectEndpoint(MIDI_STREAM_IN_EPNUM);
	
	if(Endpoint_IsINReady()) {	//Endpoint isn't ready to write, so bail out until next time
		if(trigger_input_changed){
			send_triggers_data();
			trigger_input_changed = false;
		}
	}	

	/* Select the MIDI OUT stream (OUT from host) */
	Endpoint_SelectEndpoint(MIDI_STREAM_OUT_EPNUM);

	/* Check if endpoint is ready to be read from, if so discard its (unused) data */
	if (Endpoint_IsOUTReceived()){
		Endpoint_ClearOUT();
	}

}

void send_triggers_data(){
	
	for(uint8_t bit = 0; bit < 16; bit++){
		bool was_on = (last_trigger_state & (1 << bit));
		bool is_on = (trigger_state & (1 << bit));
		if(is_on != was_on){
			SendMIDINoteChange(bit, is_on, 0, MIDI_CHANNEL);
			break;
		}
	}
}

void EVENT_USB_Device_Connect() {
	//TODO: ????
}

void EVENT_USB_Device_Disconnect() {
	//TODO: ????
}

void EVENT_USB_Device_ConfigurationChanged() {

    /* Setup MIDI stream endpoints */
    Endpoint_ConfigureEndpoint(MIDI_STREAM_OUT_EPNUM, EP_TYPE_BULK,
                                     ENDPOINT_DIR_OUT, MIDI_STREAM_EPSIZE,
                                     ENDPOINT_BANK_SINGLE);
    Endpoint_ConfigureEndpoint(MIDI_STREAM_IN_EPNUM, EP_TYPE_BULK,
                                     ENDPOINT_DIR_IN, MIDI_STREAM_EPSIZE,
                                     ENDPOINT_BANK_SINGLE);
}
